# MSC_PyRewrite

There's a plugin for rewriting core rules files (*.conf) per
[msc_pyparser](https://pypi.org/project/msc-pyparser/). It
can be enabled in [Settings], and then appears directly in
the File menu.

This plugin bundles a handful of examples, as listed in the
"Transform" dropdown. But you can also edit the code block
before running.


## Usage

In the dialog, you can limit the operation to specific
secrule *.conf files and optionally id ranges. Then there's
a dropdown for default samples, and the actual code editor.

  * So first select the files you want to have rewritten.
    They must toggled on by one.
  * Optionally set a range to limit the processing to SecRules.
    The format is `0-9999999` to defined a range. Clear the
    field if you want to apply the changes to any and all rules
    instead.
  * Then pick a transform script to apply on each rule or file.
    The selection will update/overwrite the code block input.
  * The shown scripts can be adapted before executing them.
    What you see in the editor box is what will actually run.
    So it must be valid Python code with proper indentation.

The "Run/Rewrite" button will execute that code block. Ideally
you want to leave the `--dry-run` mode enabled first, to see
if it runs into any errors. In this mode the files aren't
rewritten, but potentially changes brought up in a preview
window only.

Note that any print() output will appear on the console/terminal
where modseccfg was started from.


##  Operation modes


### Rule-by-rule

All default snippets only change properties on a rule by
rule basis. They typically utilize the `rule.function()`
convenience methods.

But you can also use the raw `r` dictionary.


### Block-wise

Alternatively, there's the block-wise mode, which might allow
some msc_parser transform functions to be used literally.
Simply prefix the code block with `# type: block` to mark up
such scripts.

Might be possible to utilize the examples/ from msc_pyparser
directly, since the Transform function is often suitable
for direct use as block-wise script.

For example the [example04_reduce_escs.py](https://github.com/digitalwave/msc_pyparser/blob/master/examples/example04_reduce_escs.py)
contains a "Transform" class. Just copy that into the input box.
And append an instantiation and inocation call:

    # type: block

    # copy verbatim
    class Transform(object):
        def __init__(self, data):
	...

    # execute
    Transform(self.data).stripescape()

Where `self.data` is the raw parser structure, and `.stripescape()`
the target method in this example/class.


### Available variables

You can see the "help" Transform sample for some notes:

   * `r` is the plain msc_pyparser dict for a rule (alias: `d`)
       * r["type"] is often "SecRule"
       * r["actions"] a list of dicts (alias: `actions`)
   * `rule` is a wrapper object with some convenience methods
       * rule.id()
       * rule.has_tag("app")
       * rule.add_tag("my_new_tag")
       * for l in rule.find_actions("logdata"):
   * `parser` is the current MSCParser instance
   * `self.data` is parser.configlines

